const readline = require("readline");
const fs = require("fs");

// 漢数字の数値化
const k2n = function(a) {
  if (a.match(/^([一二三四五六七八九十])$/)) return "一二三四五六七八九十".indexOf(a) + 1;
  if (a.match(/^([一二三四五六七八九])十$/)) return k2n(RegExp.$1) * 10;
  if (a.match(/^十([一二三四五六七八九])$/)) return 10 + k2n(RegExp.$1);
  if (a.match(/^([二三四五六七八九])十([一二三四五六七八九])$/)) {
    const v = [RegExp.$1, RegExp.$2];
    return k2n(v[0]) * 10 + k2n(v[1]);
  }
  return -1;
}

const aliases = [
  ["龍ヶ崎市", "龍ケ崎市"],
  ["須恵町", "須惠町"],
  ["外ケ浜町", "外ヶ浜町"],
  ["茅ヶ崎市", "茅ケ崎市"],
  ["塩竈市", "塩釜市", "塩竃市"],
  ["駒ヶ根市", "駒ケ根市"],
  ["鎌ヶ谷市", "鎌ケ谷市"],
  ["袖ヶ浦市", "袖ケ浦市"],
  ["保土ヶ谷区", "保土ケ谷区"]
];

const map = (function(sac) {

  sac.results.bindings.forEach(a => {
    const label = a.label.value;
    if (aliases.find(x => x.indexOf(label) !== -1)) return;
    if (label.match(/[ヶケ]/)) {
      aliases.push([label.replace(/ケ/g, "ヶ"), label.replace(/ヶ/g, "ケ")]);
      console.error(label);
    }
  });

  const sacmap = {};
  sac.results.bindings.forEach(a => {
    const o = {};
    Object.keys(a).forEach(key => {
      o[key] = a[key].value;
    });
    sacmap[o.s] = o;
    aliases.forEach(list => {
      if (list.indexOf(o.label) === -1) return;
      list.forEach(alias => {
        if (alias !== o.label) {
          const target = Object.assign({}, o);
          target.s += "_" + alias;
          target.label = alias;
          sacmap[target.s] = target;
        }
      });
    });
    /*    if (replacement[o.label] !== undefined) {
          replacement[o.label].forEach((x, i) => {
            const target = Object.assign({}, o);
            target.s += "_" + i;
            target.label = x;
            sacmap[target.s] = target;
          });
        }*/
  });

  const obj = {};

  Object.keys(sacmap).forEach(s => {
    const v = sacmap[s];
    const tokens = [];
    for (let f = v; f;) {
      if (f.label.match(/[都道府県郡市区町村]$/))
        tokens.unshift(f.label);
      if (f.gun) tokens.unshift(f.gun);
      f = sacmap[f.parent];
    }
    const key = tokens.join(",");
    if (obj[key] === undefined) obj[key] = false;
    if (v.valid === undefined) obj[key] = v.notation;
  });
  return obj;
})(JSON.parse(fs.readFileSync(process.argv[2], "UTF-8")));

Object.keys(map).forEach(key => {
  if (map[key] !== false) map[map[key]] = {};
});

readline.createInterface({
  input: process.stdin
}).on('line', (line) => {

  const row = line.replace(/"/g, "").split(",");
  const code = row[2];
  if (!code.match(/^[0-9]{5}$/)) return;

  if (map[code] === undefined) return;

  const city = row[3];
  let name = row[5];
  if (city.endsWith(name)) {
    const x = Object.keys(map).find(a => map[a] === code);
    if (x.split(",").pop() === name) {
      console.error("自治体名と町名が一致: " + row + " : " + name);
      return;
    }
  }

  let chome = "";

  if (name.match(/^(.*[^一二三四五六七八九十])([一二三四五六七八九十]+)丁目$/)) {
    name = RegExp.$1;
    chome = k2n(RegExp.$2);
  }

  if (map[code][name] === undefined) map[code][name] = [];
  if (map[code][name].indexOf(chome) === -1) map[code][name].push(chome);

}).on('close', () => {
  Object.keys(map).forEach(key => {
    if (typeof map[key] === 'string') return;
    Object.keys(map[key]).forEach(b => {

      // 抜け番の補正
      let max = NaN;
      let min = NaN;
      map[key][b].filter(x => typeof x === 'number').forEach(x => {
        max = isNaN(max) ? x : Math.max(max, x);
        min = isNaN(min) ? x : Math.min(min, x);
      });
      if (!isNaN(min) && !isNaN(max)) {
        // 1丁目からあることにする
        min = 1;
        for (let i = min; i <= max; i++) {
          if (map[key][b].find(a => a === i) === undefined) {
            //            console.error(`${b} の丁目は ${map[key][b].join(",")} ですが ${i} が抜け番`);
            map[key][b].push(i);
          }
        }
      }
      map[key][b] = map[key][b].join(",");
    });
  });
  console.log(`module.exports = ${JSON.stringify(map,null,2)};`);
});
