// 住所変換処理
const enrich = require("../main");

// メタデータの除去
const trim = (a) => {
  const dig = (f) => {
    delete f["メタデータ"];
    Object.keys(f).forEach(k => {
      const v = f[k];
      if (Array.isArray(v)) v.forEach(b => dig(b));
      else if (v && typeof v === "object") dig(v);
    });
    return f;
  };
  return dig(JSON.parse(JSON.stringify(a)));
};

// 住所変換＋メタデータ除去の一括処理
const enrich_and_trim = (a) => trim(enrich(a));

const expect = require('chai').expect;

describe('imi-enrichment-address', () => {

  describe('基本動作', () => {

    describe('正常系', () => {

      it('ルートにある住所型インスタンスの変換', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区太子堂４丁目１−１キャロットタワー"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区太子堂４丁目１−１キャロットタワー",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "太子堂",
          "丁目": "4",
          "番地": "1",
          "号": "1"
        });
      });
      it('ルートにない住所型インスタンスの変換', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "法人型",
          "表記": "株式会社ほげほげ",
          "住所": {
            "@type": "住所型",
            "表記": "東京都世田谷区太子堂４丁目１−１キャロットタワー"
          }
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "法人型",
          "表記": "株式会社ほげほげ",
          "住所": {
            "@type": "住所型",
            "表記": "東京都世田谷区太子堂４丁目１−１キャロットタワー",
            "都道府県": "東京都",
            "市区町村": "世田谷区",
            "町名": "太子堂",
            "丁目": "4",
            "番地": "1",
            "号": "1"
          }
        });
      });
      it('@graph による複数インスタンスの変換', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@graph": [{
            "@type": "住所型",
            "表記": "東京都世田谷区三軒茶屋2-11-22サンタワーズセンタービル"
          }, {
            "@type": "法人型",
            "表記": "株式会社ほげほげ",
            "住所": {
              "@type": "住所型",
              "表記": "東京都世田谷区太子堂４丁目１−１キャロットタワー"
            }
          }]
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@graph": [{
            "@type": "住所型",
            "表記": "東京都世田谷区三軒茶屋2-11-22サンタワーズセンタービル",
            "都道府県": "東京都",
            "市区町村": "世田谷区",
            "町名": "三軒茶屋",
            "丁目": "2",
            "番地": "11",
            "号": "22"
          }, {
            "@type": "法人型",
            "表記": "株式会社ほげほげ",
            "住所": {
              "@type": "住所型",
              "表記": "東京都世田谷区太子堂４丁目１−１キャロットタワー",
              "都道府県": "東京都",
              "市区町村": "世田谷区",
              "町名": "太子堂",
              "丁目": "4",
              "番地": "1",
              "号": "1"
            }
          }]
        });
      });
      it('hj:住所型インスタンスの変換', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "hj:住所型",
          "表記": "東京都世田谷区太子堂４丁目１−１キャロットタワー"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "hj:住所型",
          "表記": "東京都世田谷区太子堂４丁目１−１キャロットタワー",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "太子堂",
          "丁目": "4",
          "番地": "1",
          "号": "1"
        });
      });
    });

    describe('異常系', () => {
      it('ルートオブジェクト配下に住所型が見つからない場合は警告', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "法人型",
          "表記": "株式会社ほげほげ"
        });
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
      });
      it('@graph の直下および配下に住所型が見つからない場合は警告', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@graph": []
        });
        expect(result["メタデータ"]).is.not.empty;
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
      });
    });

    describe('JSON-LD Context の付与', () => {
      it('@context がない場合は既定値を付与する', () => {
        expect(enrich_and_trim({
          "@type": "住所型",
          "表記": "東京都"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都",
          "都道府県": "東京都"
        });
      });

      it('コアのみの場合は追加する', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都",
          "都道府県": "東京都"
        });
      });

      it('法人のみの場合は追加する', () => {
        expect(enrich_and_trim({
          "@context": "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld",
          "@type": "住所型",
          "表記": "東京都"
        })).deep.equal({
          "@context": [
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld",
            "https://imi.go.jp/ns/core/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都",
          "都道府県": "東京都"
        });
      });

      it('最初から両定義がある場合はなにもしない', () => {
        expect(enrich_and_trim({
          "@context": [
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld",
            "https://imi.go.jp/ns/core/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都"
        })).deep.equal({
          "@context": [
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld",
            "https://imi.go.jp/ns/core/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都",
          "都道府県": "東京都"
        });

        expect(enrich_and_trim({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都",
          "都道府県": "東京都"
        });
      });

      it('オブジェクト定義の場合は付与する', () => {
        expect(enrich_and_trim({
          "@context": {
            "ex": "http://example.org"
          },
          "@type": "住所型",
          "表記": "東京都"
        })).deep.equal({
          "@context": [{
              "ex": "http://example.org"
            },
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都",
          "都道府県": "東京都"
        });
      });

      it('他の context.jsonld がある場合は足りないものを追記', () => {
        expect(enrich_and_trim({
          "@context": "http://schema.org/",
          "@type": "住所型",
          "表記": "東京都"
        })).deep.equal({
          "@context": [
            "http://schema.org/",
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都",
          "都道府県": "東京都"
        });

        expect(enrich_and_trim({
          "@context": [
            "http://schema.org/",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都"
        })).deep.equal({
          "@context": [
            "http://schema.org/",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld",
            "https://imi.go.jp/ns/core/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都",
          "都道府県": "東京都"
        });
      });
    });
  });

  describe('各種の住所表記に対する対応', () => {

    describe('省略のない住所', () => {

      it('都道府県', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都",
          "都道府県": "東京都"
        });
      });

      it('都道府県＋郡', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "長野県諏訪郡"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "長野県諏訪郡",
          "都道府県": "長野県"
        });
      });

      it('都道府県＋市', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都調布市"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都調布市",
          "都道府県": "東京都",
          "市区町村": "調布市"
        });
      });

      it('都道府県＋区', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区",
          "都道府県": "東京都",
          "市区町村": "世田谷区"
        });
      });

      it('都道府県＋郡＋町', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "長野県諏訪郡下諏訪町"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "長野県諏訪郡下諏訪町",
          "都道府県": "長野県",
          "市区町村": "下諏訪町"
        });
      });

      it('都道府県＋郡＋村', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "長野県諏訪郡原村"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "長野県諏訪郡原村",
          "都道府県": "長野県",
          "市区町村": "原村"
        });
      });

      it('都道府県＋町', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都大島町"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都大島町",
          "都道府県": "東京都",
          "市区町村": "大島町"
        });
      });

      it('都道府県＋村', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都神津島村"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都神津島村",
          "都道府県": "東京都",
          "市区町村": "神津島村"
        });
      });

      it('都道府県＋市＋区', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "神奈川県横浜市青葉区"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "神奈川県横浜市青葉区",
          "都道府県": "神奈川県",
          "市区町村": "横浜市",
          "区": "青葉区"
        });
      });

      it('（自治体名）＋町名', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "三軒茶屋"
        });
      });

      it('（自治体名）＋町名＋丁目', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋2"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋2",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "三軒茶屋",
          "丁目": "2"
        });
      });

      it('（自治体名）＋町名＋丁目＋番地', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋2-11"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋2-11",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "三軒茶屋",
          "丁目": "2",
          "番地": "11"
        });
      });

      it('（自治体名）＋町名＋丁目＋番地＋号', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋2-11-22"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋2-11-22",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "三軒茶屋",
          "丁目": "2",
          "番地": "11",
          "号": "22"
        });
      });

      it('（自治体名）＋町名＋丁目＋番地＋号＋ビル名', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋2-11-22サンタワーズセンタービル"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋2-11-22サンタワーズセンタービル",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "三軒茶屋",
          "丁目": "2",
          "番地": "11",
          "号": "22"
        });
      });

      it('（自治体名）＋町名＋番地', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都西多摩郡奥多摩町氷川215"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都西多摩郡奥多摩町氷川215",
          "都道府県": "東京都",
          "市区町村": "奥多摩町",
          "町名": "氷川",
          "番地": "215"
        });
      });

      it('（自治体名）＋町名＋番地＋号', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都西多摩郡奥多摩町氷川215-6"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都西多摩郡奥多摩町氷川215-6",
          "都道府県": "東京都",
          "市区町村": "奥多摩町",
          "町名": "氷川",
          "番地": "215",
          "号": "6"
        });
      });

      it('（自治体名）＋番号', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都神津島村904番地"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都神津島村904番地",
          "都道府県": "東京都",
          "市区町村": "神津島村"
        });

        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都西多摩郡檜原村467-1"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都西多摩郡檜原村467-1",
          "都道府県": "東京都",
          "市区町村": "檜原村"
        });
      });

    });

    describe('省略のある住所', () => {

      it('候補が複数の場合は警告してデータには変更がないこと、説明中に候補地名が列挙されること', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "中央区日本橋"
        });
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
        expect(result["メタデータ"]["説明"]).to.have.string("東京都中央区");
        expect(result["メタデータ"]["説明"]).to.have.string("大阪府大阪市中央区");
        expect(trim(result)).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "中央区日本橋"
        });
      });

      it('ユニーク（町名）による都道府県市区町村の補完=>都道府県市区町村のいずれかの文字がない場合は住所扱いしない', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "三軒茶屋"
        });
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
        expect(trim(result)).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "三軒茶屋"
        });
      });

      it('ユニーク（都道府県＋町名）による市区町村の補完', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "神奈川県新宿"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "神奈川県新宿",
          "都道府県": "神奈川県",
          "市区町村": "逗子市",
          "町名": "新宿"
        });
      });

      it('ユニーク（区名＋町名）による都道府県の補完', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "港区赤坂"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "港区赤坂",
          "都道府県": "東京都",
          "市区町村": "港区",
          "町名": "赤坂"
        });
      });

      it('世田谷区桜 : 世田谷区桜新町にマッチしないこと', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "世田谷区桜"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "世田谷区桜",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "桜"
        });
      });

      it('世田谷区桜新町 : 世田谷区桜にマッチしないこと', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "世田谷区桜新町"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "世田谷区桜新町",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "桜新町"
        });
      });

    });

    describe('丁目・番地・号', () => {

      it('半角数値で記述された丁目・番地・号', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋2-11-22"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋2-11-22",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "三軒茶屋",
          "丁目": "2",
          "番地": "11",
          "号": "22"
        });
      });
      it('全角数値で記述された丁目・番地・号', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋２－１１－２２"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋２－１１－２２",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "三軒茶屋",
          "丁目": "2",
          "番地": "11",
          "号": "22"
        });
      });

      it('漢数字で記述された丁目・番地・号', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋二丁目十一番二十二号"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋二丁目十一番二十二号",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "三軒茶屋",
          "丁目": "2",
          "番地": "11",
          "号": "22"
        });
      });

      it('漢数字(〇を使用)で記述された丁目・番地・号', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋二丁目一〇番二〇号"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋二丁目一〇番二〇号",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "三軒茶屋",
          "丁目": "2",
          "番地": "10",
          "号": "20"
        });
      });

      it('混在', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋二丁目10-２０"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋二丁目10-２０",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "三軒茶屋",
          "丁目": "2",
          "番地": "10",
          "号": "20"
        });
      });

      it('下位町名', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "京都府宇治市槇島町一町田４１−１"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "京都府宇治市槇島町一町田４１−１",
          "都道府県": "京都府",
          "市区町村": "宇治市",
          "町名": "槇島町"
        });
      });

      it('合併地', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "愛知県新城市字町並８２・８３番合併地"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "愛知県新城市字町並８２・８３番合併地",
          "都道府県": "愛知県",
          "市区町村": "新城市",
          "町名": "字町並"
        });
      });

      it('合地', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "京都府京都市中京区御幸町通二条上る達磨町５９４番５９５番合地"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "京都府京都市中京区御幸町通二条上る達磨町５９４番５９５番合地",
          "都道府県": "京都府",
          "市区町村": "京都市",
          "区": "中京区",
          "町名": "達磨町"
        });
      });



    });

    describe('表記揺らぎ', () => {
      it('自治体名（小さいヶ）', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "茨城県龍ヶ崎市"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "茨城県龍ヶ崎市",
          "都道府県": "茨城県",
          "市区町村": "龍ヶ崎市"
        });

      });
      it('自治体名（大きいヶ）', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "茨城県龍ケ崎市"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "茨城県龍ケ崎市",
          "都道府県": "茨城県",
          "市区町村": "龍ケ崎市"
        });
      });
      it('既知の揺らぎ自治体名', () => {

        ["龍ヶ崎市", "龍ケ崎市",
          "須恵町", "須惠町",
          "外ケ浜町", "外ヶ浜町",
          "茅ヶ崎市", "茅ケ崎市",
          "塩竈市", "塩釜市", "塩竃市",
          "駒ヶ根市", "駒ケ根市",
          "鎌ヶ谷市", "鎌ケ谷市",
          "袖ヶ浦市", "袖ケ浦市"
        ].forEach(name => {
          const result = enrich({
            "@context": "https://imi.go.jp/ns/core/context.jsonld",
            "@type": "住所型",
            "表記": name
          });
          expect(result["市区町村"]).to.equal(name);
        });
      });
      it('町名（小さいヶ）', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都渋谷区幡ヶ谷"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都渋谷区幡ヶ谷",
          "都道府県": "東京都",
          "市区町村": "渋谷区",
          "町名": "幡ヶ谷"
        });
      });
      it('町名（大きいヶ）', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都渋谷区幡ケ谷"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都渋谷区幡ケ谷",
          "都道府県": "東京都",
          "市区町村": "渋谷区",
          "町名": "幡ケ谷"
        });
      });

      it('町名（北海道の条番号、全角数字一桁)', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "北海道札幌市豊平区豊平６条"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "北海道札幌市豊平区豊平６条",
          "都道府県": "北海道",
          "市区町村": "札幌市",
          "区": "豊平区",
          "町名": "豊平６条"
        });
      });

      it('町名（北海道の条番号、全角数字二桁)', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "北海道札幌市東区北４７条東"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "北海道札幌市東区北４７条東",
          "都道府県": "北海道",
          "市区町村": "札幌市",
          "区": "東区",
          "町名": "北４７条東"
        });
      });

      it('町名（北海道の条番号、半角数字一桁)', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "北海道札幌市豊平区豊平6条"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "北海道札幌市豊平区豊平6条",
          "都道府県": "北海道",
          "市区町村": "札幌市",
          "区": "豊平区",
          "町名": "豊平6条"
        });
      });

      it('町名（北海道の条番号、半角数字二桁)', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "北海道札幌市東区北47条東"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "北海道札幌市東区北47条東",
          "都道府県": "北海道",
          "市区町村": "札幌市",
          "区": "東区",
          "町名": "北47条東"
        });
      });
    });

    describe('ログ種別', () => {
      it('正常系', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋2-11-22"
        });
        expect(result["メタデータ"]["種別"]).to.equal("INFO");
      });

      it('過去自治体', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "埼玉県浦和市"
        });
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
      });
      it('海外住所', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "中華人民共和国"
        });
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
      });
      it('複数候補(自治体)', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "中央区"
        });
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
      });
      it('複数候補(町名)', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "中央区日本橋"
        });
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
      });
      it('都道府県の補完', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "世田谷区三軒茶屋"
        });
        expect(result["メタデータ"]["種別"]).to.equal("INFO");
      });
      it('市区町村の補完', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "三軒茶屋"
        });
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
      });
    });

    describe('異常系', () => {

      it('存在しない丁目の場合には丁目以下は付与せず警告すること＞修正:辞書が完璧ではないので、丁目は付与する', () => {
        const result = enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋22-11-22サンタワーズセンタービル"
        });
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
        expect(trim(result)).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都世田谷区三軒茶屋22-11-22サンタワーズセンタービル",
          "都道府県": "東京都",
          "市区町村": "世田谷区",
          "町名": "三軒茶屋",
          "丁目": "22",
          "番地": "11",
          "号": "22"
        });
      });

      it('過去に存在した自治体の場合は町名以下は付与しないこと', () => {
        const input = {
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "黒川郡富谷町成田二丁目２２−１"
        };
        const result = enrich(input);
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
        expect(result["メタデータ"]["説明"]).to.have.string("過去");
        expect(trim(result)).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "黒川郡富谷町成田二丁目２２−１",
          "都道府県": "宮城県",
          "市区町村": "富谷町"
        });
      });

      it('海外住所(共和国)が国内住所に変換されないこと', () => {
        const input = {
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "ドイツ連邦共和国"
        };
        const result = enrich(input);
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
        expect(trim(result)).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "ドイツ連邦共和国"
        });
      });


      it('都道府県だけがあっている地名、都道府県だけは付与すること', () => {
        const input = {
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都日野旭が丘４−７−１２７"
        };
        const result = enrich(input);
        expect(result["メタデータ"]["種別"]).to.equal("WARNING");
        expect(trim(result)).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都日野旭が丘４−７−１２７",
          "都道府県": "東京都"
        });
      });

      it('都道府県だけがあっている地名、都道府県だけは付与すること', () => {
        expect(trim(enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都南区赤坂２丁目２番１９号"
        }))).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都南区赤坂２丁目２番１９号",
          "都道府県": "東京都"
        });
      });
      /*

              ["大阪府豊能郡豊野町余野６５９", "東京都南区赤坂２丁目２番１９号", "東京都大東区東上野５−１３−１０",
                "福岡県糟屋新宮町的野香ノ木７４１−１", "秋田県仙北郡美里町金沢東根字南外川原１２６",
                "長野県東築摩郡山形村大字下本郷４０８２−３",
                "鳥取県堺港市昭和町３４"
              ].forEach()
              "*/

      it('郡名が間違っている地名に都道府県・村名が付与されること', () => {
        expect(trim(enrich({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "長野県東築摩郡山形村大字下本郷４０８２−３"
        }))).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "長野県東築摩郡山形村大字下本郷４０８２−３",
          "都道府県": "長野県",
          "市区町村": "山形村"
        });
      });

      it('海外国名混在型住所は付与すること', () => {
        const input = {
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "港区南麻布４丁目１１番４４号在日フランス大使館内"
        };
        const result = enrich(input);
        expect(trim(result)).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "港区南麻布４丁目１１番４４号在日フランス大使館内",
          "都道府県": "東京都",
          "市区町村": "港区",
          "町名": "南麻布",
          "丁目": "4",
          "番地": "11",
          "号": "44"
        });
      });



    });

    describe('デバッグ用特殊地名', () => {

      // 過去地名
      it('鹿児島県姶良郡姶良町宮島町', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "鹿児島県姶良郡姶良町宮島町"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "鹿児島県姶良郡姶良町宮島町",
          "都道府県": "鹿児島県",
          "市区町村": "姶良町"
        });
      });

      // 東京都の府中市にヒットすること
      it('東京都府中市', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都府中市"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都府中市",
          "都道府県": "東京都",
          "市区町村": "府中市"
        });
      });



      // 「市」が省略された異常系だが、町名がユニークで救済できるパターン
      it('東京都八王子滝山町２−９３−１', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都八王子滝山町２−９３−１"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都八王子滝山町２−９３−１",
          "都道府県": "東京都",
          "市区町村": "八王子市",
          "町名": "滝山町",
          "丁目": "2",
          "番地": "93",
          "号": "1"
        });
      });

      // 「鹿児島市」が省略された異常系だが、町名がユニークで救済できるパターン
      it('鹿児島県甲突町１８−２１シャルム甲突１０２号', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "鹿児島県甲突町18-21シャルム甲突１０２号"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "鹿児島県甲突町18-21シャルム甲突１０２号",
          "都道府県": "鹿児島県",
          "市区町村": "鹿児島市",
          "町名": "甲突町",
          "番地": "18",
          "号": "21"

        });
      });


      // 県名だけのパターン
      it('香川県', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "香川県"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "香川県",
          "都道府県": "香川県"
        });
      });


      // 都道府県＋郡だけのパターン
      it('長野県諏訪郡', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "長野県諏訪郡"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "長野県諏訪郡",
          "都道府県": "長野県"
        });
      });

      // 郡だけのパターン＝＞住所文字列として判定しない
      it('諏訪郡', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "諏訪郡"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "諏訪郡"
        });
      });

      // ユニーク市名に間違った都道府県がついた事例
      it('沖縄県奄美市名瀬入舟町', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "沖縄県奄美市名瀬入舟町"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "沖縄県奄美市名瀬入舟町",
          "都道府県": "鹿児島県",
          "市区町村": "奄美市",
          "町名": "名瀬入舟町"
        });
      });

      // 市区町村の「ヶ」「ケ」の別名は辞書登録によって対処する
      it('茨城県龍ヶ崎市佐貫', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "茨城県龍ヶ崎市佐貫"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "茨城県龍ヶ崎市佐貫",
          "都道府県": "茨城県",
          "市区町村": "龍ヶ崎市",
          "町名": "佐貫"
        });
      });

      //「都」が旧字体の事例、しかし「中央区八丁堀」で救済できた
      it('東京都中央区八丁堀一丁目１１番３号', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都中央区八丁堀一丁目１１番３号"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都中央区八丁堀一丁目１１番３号",
          "都道府県": "東京都",
          "市区町村": "中央区",
          "町名": "八丁堀",
          "丁目": "1",
          "番地": "11",
          "号": "3"
        });
      });

      // 東京都港区がヒットすること
      it('港区浜松町', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "港区浜松町"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "港区浜松町",
          "都道府県": "東京都",
          "市区町村": "港区",
          "町名": "浜松町"
        });
      });

      // 合併による合成町名の例
      it('高松市国分寺町福家字下福家甲３８７１番地３', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "高松市国分寺町福家字下福家甲３８７１番地３"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "高松市国分寺町福家字下福家甲３８７１番地３",
          "都道府県": "香川県",
          "市区町村": "高松市",
          "町名": "国分寺町福家"
        });
      });

      // 候補が複数ある事例
      it('府中市府中町', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "府中市府中町"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "府中市府中町"
        });
      });

      // 市区町村名の誤字。四條畷市 なので誤字、しかし楠公がユニークなので対応できるた事例
      it('大阪府四条畷市楠公', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "大阪府四条畷市楠公"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "大阪府四条畷市楠公",
          "都道府県": "大阪府",
          "市区町村": "四條畷市",
          "町名": "楠公"
        });
      });

      // 四條畷市 なので誤字、しかし都道府県名だけは拾えるべし
      it('大阪府四条畷市', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "大阪府四条畷市"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "大阪府四条畷市",
          "都道府県": "大阪府"
        });
      });

      // 江差町と思われるので、誤字。ただし字伏木戸町がユニークなので対応できた
      it('北海道桧山郡江別町字伏木戸町', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "北海道桧山郡江別町字伏木戸町"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "北海道桧山郡江別町字伏木戸町",
          "都道府県": "北海道",
          "市区町村": "江差町",
          "町名": "字伏木戸町"
        });
      });


      // 実際に二つあるのでしかたない
      it('中央区日本橋', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "中央区日本橋"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "中央区日本橋"
        });
      });

      // 政令指定市の区配下のパース：当初「道場」だったがデータの誤りで「道場北」なら OK
      it('千葉市中央区道場北１−９−１５', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "千葉市中央区道場北１−９−１５"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "千葉市中央区道場北１−９−１５",
          "都道府県": "千葉県",
          "市区町村": "千葉市",
          "区": "中央区",
          "町名": "道場北",
          "丁目": "1",
          "番地": "9",
          "号": "15"
        });
      });

      it('北海道札幌市中央区北１条東１', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "北海道札幌市中央区北１条東１"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "北海道札幌市中央区北１条東１",
          "都道府県": "北海道",
          "市区町村": "札幌市",
          "区": "中央区",
          "町名": "北１条東",
          "丁目": "1"
        });
      });

      it('北海道札幌市中央区南１条西１', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "北海道札幌市中央区南１条西１"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "北海道札幌市中央区南１条西１",
          "都道府県": "北海道",
          "市区町村": "札幌市",
          "区": "中央区",
          "町名": "南１条西",
          "丁目": "1"
        });
      });

      it('北海道網走郡美幌町字東２条北２', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "北海道網走郡美幌町字東２条北２"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "北海道網走郡美幌町字東２条北２",
          "都道府県": "北海道",
          "市区町村": "美幌町",
          "町名": "字東２条北",
          "丁目": "2"
        });
      });

      // 横浜「市」がない事例。神奈川県（横浜市）中区,静岡県（浜松市）中区 に該当してしまうが、神奈川県に寄せたい
      it('神奈川県横浜中区山下町１番地　シルクビル２Ｆ', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "神奈川県横浜中区山下町１番地　シルクビル２Ｆ"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "神奈川県横浜中区山下町１番地　シルクビル２Ｆ",
          "都道府県": "神奈川県",
          "市区町村": "横浜市",
          "区": "中区",
          "町名": "山下町",
          "番地": "1"
        });
      });

      // 後続の「四谷」 に引っ張られている
      it('東京都新宿区若葉１丁目８番地６号メサ四谷３０１', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都新宿区若葉１丁目８番地６号メサ四谷３０１"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都新宿区若葉１丁目８番地６号メサ四谷３０１",
          "都道府県": "東京都",
          "市区町村": "新宿区",
          "町名": "若葉",
          "丁目": "1",
          "番地": "8",
          "号": "6"
        });
      });

      // 丁目が必須なのに省略されているケース、１０丁目にならないようにしたい
      it('宮城県仙台市若林区遠見塚１０番７号', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "宮城県仙台市若林区遠見塚１０番７号"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "宮城県仙台市若林区遠見塚１０番７号",
          "都道府県": "宮城県",
          "市区町村": "仙台市",
          "区": "若林区",
          "町名": "遠見塚",
          "番地": "10",
          "号": "7"
        });
      });

      // 千代田区だが「区」がないために町名の「千代田」にマッチしている
      // 救済が難しいので東京都千代田区千代田にしたい
      it('東京都千代田内幸町二丁目２番３号', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "東京都千代田内幸町二丁目２番３号"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "東京都千代田内幸町二丁目２番３号",
          "都道府県": "東京都",
          "市区町村": "千代田区",
          "町名": "千代田"
        });
      });

      it('京都市東山区大和大路五条下ル東入芳野町７９番地の２', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "京都市東山区大和大路五条下ル東入芳野町７９番地の２"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "京都市東山区大和大路五条下ル東入芳野町７９番地の２",
          "都道府県": "京都府",
          "市区町村": "京都市",
          "区": "東山区",
          "町名": "芳野町",
          "番地": "79",
          "号": "2"
        });
      });

      it('一関市滝沢字鶴ケ沢７−６８', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "一関市滝沢字鶴ケ沢７−６８"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "一関市滝沢字鶴ケ沢７−６８",
          "都道府県": "岩手県",
          "市区町村": "一関市",
          "町名": "滝沢"
        });
      });

      it('三重県名張市すずらん台西１番町３４１', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "三重県名張市すずらん台西１番町３４１"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "三重県名張市すずらん台西１番町３４１",
          "都道府県": "三重県",
          "市区町村": "名張市",
          "町名": "すずらん台西１番町",
          "番地": "341"
        });
      });

      it('千葉県千葉市富士見２−９−２８', () => {
        expect(enrich_and_trim({
          "@context": "https://imi.go.jp/ns/core/context.jsonld",
          "@type": "住所型",
          "表記": "千葉県千葉市富士見２−９−２８"
        })).deep.equal({
          "@context": [
            "https://imi.go.jp/ns/core/context.jsonld",
            "http://hojin-info.go.jp/ns/domain/biz/1/context.jsonld"
          ],
          "@type": "住所型",
          "表記": "千葉県千葉市富士見２−９−２８",
          "都道府県": "千葉県",
          "市区町村": "千葉市",
          "区": "中央区",
          "町名": "富士見",
          "丁目": "2",
          "番地": "9",
          "号": "28"
        });
      });



    });

  });
});
