const util = require("../lib/util");
const expect = require('chai').expect;

describe('util', function() {
  describe('z2h', function() {
    it('1桁の全角数字', function() {
      expect(util.z2h("０")).to.equal("0");
      expect(util.z2h("１")).to.equal("1");
      expect(util.z2h("２")).to.equal("2");
      expect(util.z2h("３")).to.equal("3");
      expect(util.z2h("４")).to.equal("4");
      expect(util.z2h("５")).to.equal("5");
      expect(util.z2h("６")).to.equal("6");
      expect(util.z2h("７")).to.equal("7");
      expect(util.z2h("８")).to.equal("8");
      expect(util.z2h("９")).to.equal("9");
    });
    it('2桁の全角数字', function() {
      expect(util.z2h("１０")).to.equal("10");
      expect(util.z2h("１１")).to.equal("11");
      expect(util.z2h("１２")).to.equal("12");
      expect(util.z2h("１３")).to.equal("13");
      expect(util.z2h("１４")).to.equal("14");
      expect(util.z2h("１５")).to.equal("15");
      expect(util.z2h("１６")).to.equal("16");
      expect(util.z2h("１７")).to.equal("17");
      expect(util.z2h("１８")).to.equal("18");
      expect(util.z2h("１９")).to.equal("19");

      expect(util.z2h("２０")).to.equal("20");
      expect(util.z2h("２１")).to.equal("21");
      expect(util.z2h("２２")).to.equal("22");
      expect(util.z2h("２３")).to.equal("23");
      expect(util.z2h("２４")).to.equal("24");
      expect(util.z2h("２５")).to.equal("25");
      expect(util.z2h("２６")).to.equal("26");
      expect(util.z2h("２７")).to.equal("27");
      expect(util.z2h("２８")).to.equal("28");
      expect(util.z2h("２９")).to.equal("29");
    });
  });
  describe('h2k', function() {
    it('1桁の半角数字', function() {
      expect(util.h2k("0")).to.equal("〇");
      expect(util.h2k("1")).to.equal("一");
      expect(util.h2k("2")).to.equal("二");
      expect(util.h2k("3")).to.equal("三");
      expect(util.h2k("4")).to.equal("四");
      expect(util.h2k("5")).to.equal("五");
      expect(util.h2k("6")).to.equal("六");
      expect(util.h2k("7")).to.equal("七");
      expect(util.h2k("8")).to.equal("八");
      expect(util.h2k("9")).to.equal("九");
    });

    it('2桁の半角数字', function() {
      expect(util.h2k("10")).to.equal("十");
      expect(util.h2k("11")).to.equal("十一");
      expect(util.h2k("12")).to.equal("十二");
      expect(util.h2k("13")).to.equal("十三");
      expect(util.h2k("14")).to.equal("十四");
      expect(util.h2k("15")).to.equal("十五");
      expect(util.h2k("16")).to.equal("十六");
      expect(util.h2k("17")).to.equal("十七");
      expect(util.h2k("18")).to.equal("十八");
      expect(util.h2k("19")).to.equal("十九");

      expect(util.h2k("20")).to.equal("二十");
      expect(util.h2k("21")).to.equal("二十一");
      expect(util.h2k("22")).to.equal("二十二");
      expect(util.h2k("23")).to.equal("二十三");
      expect(util.h2k("24")).to.equal("二十四");
      expect(util.h2k("25")).to.equal("二十五");
      expect(util.h2k("26")).to.equal("二十六");
      expect(util.h2k("27")).to.equal("二十七");
      expect(util.h2k("28")).to.equal("二十八");
      expect(util.h2k("29")).to.equal("二十九");
    });

    it('3桁の半角数字', function() {
      expect(util.h2k("100")).to.equal("百");
      expect(util.h2k("101")).to.equal("百一");
      expect(util.h2k("110")).to.equal("百十");
      expect(util.h2k("111")).to.equal("百十一");

      expect(util.h2k("200")).to.equal("二百");
      expect(util.h2k("201")).to.equal("二百一");
      expect(util.h2k("210")).to.equal("二百十");
      expect(util.h2k("220")).to.equal("二百二十");
      expect(util.h2k("211")).to.equal("二百十一");
      expect(util.h2k("221")).to.equal("二百二十一");

    });
  });

  describe('k2h', function() {
    it('1桁の漢数字', function() {
      expect(util.k2h("〇")).to.equal("0");
      expect(util.k2h("一")).to.equal("1");
      expect(util.k2h("二")).to.equal("2");
      expect(util.k2h("三")).to.equal("3");
      expect(util.k2h("四")).to.equal("4");
      expect(util.k2h("五")).to.equal("5");
      expect(util.k2h("六")).to.equal("6");
      expect(util.k2h("七")).to.equal("7");
      expect(util.k2h("八")).to.equal("8");
      expect(util.k2h("九")).to.equal("9");
    });

    it('「〇」を使った2桁の漢字', function() {
      expect(util.k2h("一〇")).to.equal("10");
      expect(util.k2h("一一")).to.equal("11");
      expect(util.k2h("一二")).to.equal("12");
      expect(util.k2h("一三")).to.equal("13");
      expect(util.k2h("一四")).to.equal("14");
      expect(util.k2h("一五")).to.equal("15");
      expect(util.k2h("一六")).to.equal("16");
      expect(util.k2h("一七")).to.equal("17");
      expect(util.k2h("一八")).to.equal("18");
      expect(util.k2h("一九")).to.equal("19");

      expect(util.k2h("二〇")).to.equal("20");
      expect(util.k2h("二一")).to.equal("21");
      expect(util.k2h("二二")).to.equal("22");
      expect(util.k2h("二三")).to.equal("23");
      expect(util.k2h("二四")).to.equal("24");
      expect(util.k2h("二五")).to.equal("25");
      expect(util.k2h("二六")).to.equal("26");
      expect(util.k2h("二七")).to.equal("27");
      expect(util.k2h("二八")).to.equal("28");
      expect(util.k2h("二九")).to.equal("29");

    });

    it('「十」を使った2桁の漢字', function() {
      expect(util.k2h("十")).to.equal("10");
      expect(util.k2h("十一")).to.equal("11");
      expect(util.k2h("十二")).to.equal("12");
      expect(util.k2h("十三")).to.equal("13");
      expect(util.k2h("十四")).to.equal("14");
      expect(util.k2h("十五")).to.equal("15");
      expect(util.k2h("十六")).to.equal("16");
      expect(util.k2h("十七")).to.equal("17");
      expect(util.k2h("十八")).to.equal("18");
      expect(util.k2h("十九")).to.equal("19");

      expect(util.k2h("二十")).to.equal("20");
      expect(util.k2h("二十一")).to.equal("21");
      expect(util.k2h("二十二")).to.equal("22");
      expect(util.k2h("二十三")).to.equal("23");
      expect(util.k2h("二十四")).to.equal("24");
      expect(util.k2h("二十五")).to.equal("25");
      expect(util.k2h("二十六")).to.equal("26");
      expect(util.k2h("二十七")).to.equal("27");
      expect(util.k2h("二十八")).to.equal("28");
      expect(util.k2h("二十九")).to.equal("29");

    });

    it('「〇」を使った3桁の漢字', function() {
      expect(util.k2h("一〇〇")).to.equal("100");
      expect(util.k2h("一〇一")).to.equal("101");
      expect(util.k2h("一一〇")).to.equal("110");
      expect(util.k2h("一一一")).to.equal("111");
    });

    it('「百」「十」を使った3桁の漢字', function() {
      expect(util.k2h("百")).to.equal("100");
      expect(util.k2h("百一")).to.equal("101");
      expect(util.k2h("百十")).to.equal("110");
      expect(util.k2h("百十一")).to.equal("111");
      expect(util.k2h("百二")).to.equal("102");
      expect(util.k2h("百二十")).to.equal("120");
      expect(util.k2h("百二十二")).to.equal("122");
      expect(util.k2h("二百")).to.equal("200");
      expect(util.k2h("二百一")).to.equal("201");
      expect(util.k2h("二百十")).to.equal("210");
      expect(util.k2h("二百十一")).to.equal("211");
      expect(util.k2h("二百二")).to.equal("202");
      expect(util.k2h("二百二十")).to.equal("220");
      expect(util.k2h("二百二十二")).to.equal("222");
    });

  });
});
